
import click
import os
import json
from pathlib import Path
import re


from .spider import Spider
from .archive import Archive


# following example here:
# https://github.com/pallets/click/blob/main/examples/repo/repo.py
#
# click.secho('Hello World!', fg='green')
# click.secho('Some more text', bg='blue', fg='white')
# click.secho('ATTENTION', blink=True, bold=True)


pass_archive = click.make_pass_decorator(Archive)


@click.group()
@click.option(
    "--archive-root",
    envvar="SELVAGE_ARCHIVE_ROOT",
    default=".",
    metavar="ROOT",
    help="Changes the archive root folder location.",
    type=click.Path(path_type=Path),
)
@click.option("--verbose", "-v", is_flag=True, help="Enables verbose mode.")
@click.version_option("1.0")
@click.pass_context
def cli(ctx, archive_root, verbose):
    ctx.obj = Archive(archive_root.absolute())
    ctx.obj.verbose = verbose


@cli.command()
@pass_archive
def init(archive):
    """Initialize a new selvage archive."""
    print(f"Archive: {archive}")

@cli.command()
@pass_archive
def status(archive):
    """Report on the current status of the archive."""
    print(f"Archive: {archive}")

@cli.command()
@click.argument("input")
@click.argument("output")
@pass_archive
def add(
    archive,
    input,
    output,
):
    """Add remote resources to the archive"""
    archive.add_pattern(input, output)

@cli.command()
@click.argument("output")
@pass_archive
def output(
    archive,
    output,
):
    """Set the output"""
    archive.set_setting("output", output)

@cli.command()
@click.argument("encoding")
@pass_archive
def encoding(
    archive,
    encoding,
):
    """Set the encoding"""
    archive.set_setting("encoding", encoding)

@cli.command()
@click.option(
    "--skip-existing-files/--replace-existing-files",
    help="skip existing files",
    default=True,
)
@click.option("--verbose", is_flag=True)
@click.option("--preserve-query", is_flag=True)
@click.option(
    "-aa",
    "--additional-attribute",
    multiple=True,
    help="attributes with URLs to also consider for rewriting",
)
@click.argument("url")
@click.option("--pattern", nargs=2, type=click.Tuple([str, str]), multiple=True)
@click.option("--sleep", type=float)
@pass_archive
def spider(
    archive,
    skip_existing_files,
    verbose,
    preserve_query,
    additional_attribute,
    url,
    pattern,
    sleep,
):
    """Add remote resources to the archive"""
    s = Spider(
        archive.get_setting("output", "."),
        skip_existing_files,
        verbose=verbose,
        preserve_query=preserve_query,
        additional_attributes=additional_attribute,
        encoding=archive.get_setting("encoding")
    )
    click.echo(f"spider {s}")
    for search, replace in archive.patterns():
        print (f"pattern {search} => {replace}")
        s.add_pattern("^"+re.escape(search)+"$", replace)
    s.spider(url, sleep)

from .waybackrepair import wayback_repair

@cli.command()
@click.argument("path")
@click.option("-o", "--output-path", help="output path", default=".")
def wayback(path, output_path):
    print (f"wayback {path}")
    wayback_repair(path, output_path)    

def main() -> None:
    # print("Hello from selvage!", Spider)
    cli()
